﻿Imports System.Runtime.InteropServices

Module Module_Tweaks

    ' =====================================================================================================
    '  SystemParametersInfo
    '  https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-systemparametersinfoa
    '  When the dcumentation says to set TRUE or FALSE in the lpvParam, use 1 or 0 in the uParam
    ' =====================================================================================================
    <DllImport("user32", EntryPoint:="SystemParametersInfo", CharSet:=CharSet.Auto, SetLastError:=True)> _
    Private Function SystemParametersInfo(ByVal uAction As Integer, _
                                          ByVal uParam As Integer, _
                                          ByVal lpvParam As String, _
                                          ByVal fuWinIni As Integer) As Integer
    End Function

    <DllImport("user32", EntryPoint:="SystemParametersInfo", CharSet:=CharSet.Auto, SetLastError:=True)> _
    Private Function SystemParametersInfo(ByVal uAction As Integer, _
                                          ByVal uParam As Integer, _
                                          ByRef lpvParam As Integer, _
                                          ByVal fuWinIni As Integer) As Integer
    End Function

    Private Const SPIF_UPDATEINIFILE As Int32 = &H1
    Private Const SPIF_SENDCHANGE As Int32 = &H2

    Private Const SPI_SETWINARRANGING As Int32 = &H83    ' pvParam TRUE or FALSE
    Private Const SPI_SETDOCKMOVING As Int32 = &H91      ' pvParam TRUE or FALSE
    Private Const SPI_SETDESKWALLPAPER As Int32 = &H14   ' pvParam = String with the image path

    Private Const SPI_SETCLEARTYPE As Int32 = &H1049     ' pvParam TRUE or FALSE
    Private Const SPI_SETDROPSHADOW As Int32 = &H1025    ' pvParam TRUE or FALSE
    Private Const SPI_SETFLATMENU As Int32 = &H1023      ' pvParam TRUE or FALSE
    Private Const SPI_SETFONTSMOOTHING As Int32 = &H1025 ' uiParam TRUE or FALSE
    Private Const SPI_SETKEYBOARDDELAY As Int32 = &H17   ' uiParam = 0 (250mS) to 3 (1sec) 
    Private Const SPI_SETKEYBOARDSPEED As Int32 = &H8    ' uiParam = 0 (2.5 per second) to 31 (30 per second)
    Private Const SPI_SETMOUSESPEED As Int32 = &H71      ' pvParam = 1 (slowest) to 20 (fastest) default=10 
    Private Const SPI_SETMOUSETRAILS As Int32 = &H5D     ' uiParam = 0 (disabled) or 1 or more (number of cursors drawn in the trail)

    Private Const SPI_SETTHREADLOCALINPUTSETTINGS As Int32 = &H104F ' pvParam = TRUE (active input local per thread) or FALSE (active input global)

    Private Const SPI_SETWHEELSCROLLCHARS As Int32 = &H6D ' uiParam = number of characters to scroll when the horizontal mouse wheel is moved
    Private Const SPI_SETWHEELSCROLLLINES As Int32 = &H69 ' uiParam = number of lines to scroll when the vertical mouse wheel is moved

    Private Const SPI_SETMOUSEWHEELROUTING As Int32 = &H201D
    ' Sets the routing setting for wheel button input 
    ' The routing setting determines whether wheel button input is sent to: 
    '   - the app with focus (foreground) 
    '   - or the app under the mouse cursor
    ' The pvParam parameter must point to a DWORD variable that receives the routing option
    ' If the value is zero or MOUSEWHEEL_ROUTING_FOCUS, mouse wheel input is delivered to the app with focus
    ' If the value is 1 or MOUSEWHEEL_ROUTING_HYBRID (default): 
    '   - mouse wheel input is delivered to the app with focus (desktop apps) 
    '   - or the app under the mouse cursor (Windows Store apps)
    ' Set the uiParam parameter to zero

    Private Const SPI_SETSCREENSAVEACTIVE As Int32 = &H11 ' uiParam = TRUE or FALSE
    Private Const SPI_SETSCREENSAVETIMEOUT As Int32 = &HF ' uiParam = number of seconds of sistem idle before the screen saver activates

    Private Const SPI_SETCURSORSHADOW As Int32 = &H101B ' pvParam = TRUE or FALSE
    Private Const SPI_SETGRADIENTCAPTIONS As Int32 = &H1009 ' pvParam = TRUE or FALSE
    Private Const SPI_SETHOTTRACKING As Int32 = &H100F ' pvParam = TRUE or FALSE
    Private Const SPI_SETLISTBOXSMOOTHSCROLLING As Int32 = &H1007 ' pvParam = TRUE or FALSE

    Private Const SPI_SETACTIVEWINDOWTRACKING As Int32 = &H1001 ' pvParam = TRUE or FALSE
    Private Const SPI_SETACTIVEWNDTRKZORDER As Int32 = &H2003 ' pvParam = TRUE or FALSE



    ' =======================================================================================
    '  SHOW ALL ICONS IN THE TRAY AREA (THE APPLICATIONS BAR)
    ' =======================================================================================
    ' KEY [HKEY_CURRENT_USER \ Software \ Microsoft \ Windows \ CurrentVersion \ Explorer]
    ' Value: EnableAutoTray
    ' Data type: REG_DWORD
    ' Dati value: (0 = show inactive icons, 1 = hide inactive icons)
    '
    ' Disconnect and reconnect user or restart windows
    ' =======================================================================================
    Friend Sub ShowAllIcons()
        Dim key As String = "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer"
        Dim name As String = "EnableAutoTray"
        My.Computer.Registry.SetValue(key, name, 0, Microsoft.Win32.RegistryValueKind.DWord)
        ShowMessage(" Operation completed succesfully." + vbCrLf + _
                    " The tray area will show all the icons.")
        RestartExplorerQuestion()
    End Sub
    'Friend Sub HideInactiveIcons()
    '    Dim key As String = "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer"
    '    Dim name As String = "EnableAutoTray"
    '    My.Computer.Registry.SetValue(key, name, 1, Microsoft.Win32.RegistryValueKind.DWord)
    '    ShowMessage(" Operation completed succesfully." + vbCrLf + _
    '                " The tray area will NOT show all the icons.")
    '    RestartExplorerQuestion()
    'End Sub

    ' =======================================================================================
    '  DO NOT ADD THE "LINK" KEYWORD WHEN CREATING NEW LINKS
    ' =======================================================================================
    ' -------------------
    ' EXAMPLE: LINK OFF 
    ' -------------------
    '[HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer]
    '"link"=hex:00,00,00,00
    '
    ' -------------------
    ' EXAMPLE: LINK ON
    ' -------------------
    '[HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer]
    '"link"=hex:1E,00,00,00
    '
    ' Disconnect and reconnect user or restart windows
    ' =======================================================================================
    Friend Sub LinkKeyword_OFF()
        Dim key As String = "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer"
        Dim name As String = "link"
        Dim binValue As Byte() = {&H0, &H0, &H0, &H0}
        My.Computer.Registry.SetValue(key, name, binValue, Microsoft.Win32.RegistryValueKind.Binary)
        ShowMessage(" Operation completed succesfully." + vbCrLf + _
                    " The new links will be created without the ""link"" keyword.")
        RestartExplorerQuestion()
    End Sub
    'Friend Sub LinkKeyword_ON()
    '    Dim key As String = "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer"
    '    Dim name As String = "link"
    '    Dim binValue As Byte() = {&H1E, &H0, &H0, &H0}
    '    My.Computer.Registry.SetValue(key, name, binValue, Microsoft.Win32.RegistryValueKind.Binary)
    '    ShowMessage(" Operation completed succesfully." + vbCrLf + _
    '                " The new links will be created with the ""link"" keyword")
    '    RestartExplorerQuestion()
    'End Sub

    ' =====================================================================================================
    '  DISABLE WINDOWS DOCKING AND SHAKE
    '  https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-systemparametersinfoa
    '  SPI_SETWINARRANGING
    ' =====================================================================================================
    Friend Sub DisableWindowsArranging()
        ' ----------------------------------------------------- Disable Windows arranging & Update The Windows Setting
        SystemParametersInfo(SPI_SETWINARRANGING, _
                             0, _
                             0, _
                             SPIF_UPDATEINIFILE Or SPIF_SENDCHANGE)
        ShowMessage(" Operation completed succesfully." + vbCrLf + vbCrLf + _
                    " Now windows will not be docked when dragging them to the desktop borders." + _
                    " And when a window is shaked all the other windows are unchanged.")
    End Sub
    'Friend Sub EnableWindowsArranging()
    '    ' ----------------------------------------------------- Enable Windows arranging & Update The Windows Setting
    '    SystemParametersInfo(SPI_SETWINARRANGING, _
    '                         1, _
    '                         0, _
    '                         SPIF_UPDATEINIFILE Or SPIF_SENDCHANGE)
    '    ShowMessage(" Operation completed succesfully." + vbCrLf + vbCrLf + _
    '                " Now windows will be docked when dragging them to the desktop borders." + _
    '                " And when a window is shaked all the other windows are minimized.")
    'End Sub



    ' =======================================================================================
    '  DISABLE ACRYLIC BACKGROUND ON LOGON (access HKEY_LOCAL_MACHINE needs to be supervisor) 
    ' =======================================================================================
    '[HKEY_LOCAL_MACHINE\SOFTWARE\Policies\Microsoft\Windows\System]
    '"DisableAcrylicBackgroundOnLogon"=dword:00000001
    Friend Sub DisableAcrylicBackground()
        RestartArguments = "DisableAcrylicBackground"
        If Not EnsureAdminPrivileges() Then Return
        Dim key As String = "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\Microsoft\Windows\System"
        Dim name As String = "DisableAcrylicBackgroundOnLogon"
        My.Computer.Registry.SetValue(key, name, 1, Microsoft.Win32.RegistryValueKind.DWord)
        ShowMessage(" Operation completed succesfully." + vbCrLf + vbCrLf + _
                    " Now the blur effect at logon is disabled.")
    End Sub
    'Friend Sub EnableAcrylicBackground()
    '    RestartArguments = "EnableAcrylicBackground"
    '    If Not EnsureAdminPrivileges() Then Return
    '    Dim key As String = "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\Microsoft\Windows\System"
    '    Dim name As String = "DisableAcrylicBackgroundOnLogon"
    '    My.Computer.Registry.SetValue(key, name, 0, Microsoft.Win32.RegistryValueKind.DWord)
    '    ShowMessage(" Operation completed succesfully." + vbCrLf + vbCrLf + _
    '                " Now the blur effect at logon is enabled.")
    'End Sub

    ' ======================================================================================================
    '  Restore_Old_ImageVisualizer_JPG_PNG_GIF_BMP  (access HKEY_CLASSES_ROOT needs to be supervisor) 
    ' ======================================================================================================
    ' https://docs.microsoft.com/en-us/windows/win32/sysinfo/registry-value-types
    ' https://docs.microsoft.com/en-us/windows/win32/api/processenv/nf-processenv-expandenvironmentstringsa
    ' ======================================================================================================
    'Friend Sub RestoreOldVisualizer()
    '    RestartArguments = "RestoreOldVisualizer"
    '    If Not EnsureAdminPrivileges() Then Return
    '    RestoreOldVisualizer("jpegfile")
    '    RestoreOldVisualizer("pngfile")
    '    RestoreOldVisualizer("giffile")
    '    RestoreOldVisualizer("Paint.Picture")
    '    ShowMessage(" Operation completed succesfully." + vbCrLf + vbCrLf + _
    '                " The old image visualizer for JPG, PNG, GIF and BMP is restored.")
    'End Sub
    'Private Sub RestoreOldVisualizer(ByVal filetype As String)
    '    Dim valueOpen As String = "@%ProgramFiles%\Windows Photo Viewer\photoviewer.dll,-3043"
    '    My.Computer.Registry.SetValue("HKEY_CLASSES_ROOT\" + filetype + "\shell\open", "MuiVerb", valueOpen, Microsoft.Win32.RegistryValueKind.ExpandString)
    '    '
    '    Dim valueCommand As String = "%SystemRoot%\System32\rundll32.exe ""%ProgramFiles%\Windows Photo Viewer\PhotoViewer.dll"", ImageView_Fullscreen %1"
    '    My.Computer.Registry.SetValue("HKEY_CLASSES_ROOT\" + filetype + "\shell\open\command", "", valueCommand, Microsoft.Win32.RegistryValueKind.ExpandString)
    'End Sub

    'Friend Sub RemoveOldVisualizer()
    '    RestartArguments = "RemoveOldVisualizer"
    '    If Not EnsureAdminPrivileges() Then Return
    '    RemoveOldVisualizer("jpegfile")
    '    RemoveOldVisualizer("pngfile")
    '    RemoveOldVisualizer("giffile")
    '    RemoveOldVisualizer("Paint.Picture")
    '    ShowMessage(" Operation completed succesfully." + vbCrLf + vbCrLf + _
    '                " The old image visualizer for JPG, PNG, GIF and BMP is removed.")
    'End Sub
    'Private Sub RemoveOldVisualizer(ByVal filetype As String)
    '    Dim valueOpen As String = "@%ProgramFiles%\Windows Photo Viewer\photoviewer.dll,-3043"
    '    My.Computer.Registry.SetValue("HKEY_CLASSES_ROOT\" + filetype + "\shell\open", "MuiVerb", "", Microsoft.Win32.RegistryValueKind.ExpandString)
    '    '
    '    Dim valueCommand As String = "%SystemRoot%\System32\rundll32.exe ""%ProgramFiles%\Windows Photo Viewer\PhotoViewer.dll"", ImageView_Fullscreen %1"
    '    My.Computer.Registry.SetValue("HKEY_CLASSES_ROOT\" + filetype + "\shell\open\command", "", "", Microsoft.Win32.RegistryValueKind.ExpandString)
    'End Sub


    ' =====================================================================================================
    '  CHANGE WALLPAPER
    '  https://www.codeguru.com/columns/vb/using-visual-basic-to-change-screen-and-desktop-settings.html
    ' =====================================================================================================
    Private Function PicturesPath() As String
        ' ------------------------------------------------------------------------ returns C:\Users\xxxx\Pictures
        Return Environment.GetFolderPath(Environment.SpecialFolder.MyPictures)
    End Function

    Private Sub CopyImagesFromResources()
        My.Resources.Wallpaper1.Save(PicturesPath() + "\Wallpaper1.jpg", Imaging.ImageFormat.Jpeg)
        My.Resources.Wallpaper2.Save(PicturesPath() + "\Wallpaper2.jpg", Imaging.ImageFormat.Jpeg)
        My.Resources.Wallpaper3.Save(PicturesPath() + "\Wallpaper3.jpg", Imaging.ImageFormat.Jpeg)
    End Sub

    Friend Sub SetWallpaper(ByVal name As String)

        CopyImagesFromResources()

        If Not IO.File.Exists(name) Then name = PicturesPath() + "\" + name
        ' ----------------------------------------------------- Set Parameters To Change The Wallpaper & To Update The Windows Setting
        SystemParametersInfo(SPI_SETDESKWALLPAPER, _
                             0, _
                             name, _
                             SPIF_UPDATEINIFILE Or SPIF_SENDCHANGE)
    End Sub

    Friend Sub ChooseWallpaper()
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.InitialDirectory = PicturesPath()
        ofd.Filter = "Image files (*.jpeg;*.gif;*.png;*.bmp;*.tiff)|*.jpeg;*.jpg;*.gif;*.png;*.bmp;*.tiff;*.tif|All files (*.*)|*.*"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            If IO.File.Exists(ofd.FileName) Then
                SetWallpaper(ofd.FileName)
            End If
        End If
    End Sub


    ' =======================================================================================
    '  MESSAGES
    ' =======================================================================================
    Private Sub ShowMessage(ByVal message As String)
        MessageBox.Show(message, _
                        "Message from WinHelper", _
                        MessageBoxButtons.OK, _
                        MessageBoxIcon.Exclamation, _
                        MessageBoxDefaultButton.Button1, _
                        CType(&H40000, MessageBoxOptions))
    End Sub

    Private Sub RestartExplorerQuestion()
        If MessageBox.Show(" To complete this operation:" + vbCrLf + vbCrLf + _
                           "  - Select ""YES"" to close and restart the ""Explorer""." + vbCrLf + vbCrLf + _
                           "  - Select ""NO"" to restart the computer in a second time.", _
                           "Message from WinHelper", _
                           MessageBoxButtons.YesNo, _
                           MessageBoxIcon.Exclamation, _
                           MessageBoxDefaultButton.Button1, _
                           CType(&H40000, MessageBoxOptions)) = DialogResult.Yes Then
            RestartExplorer()
        End If
    End Sub

    ' =====================================================================================================
    '  RESTART "Explorer.exe"
    ' =====================================================================================================
    Private Sub RestartExplorer()
        For Each p As Process In Process.GetProcesses()
            ' ---------------------------------------------------------- test ProcessName for fast execution
            If Not p.ProcessName.ToLower = "explorer" Then
                Continue For
            End If
            ' ---------------------------------------------------------- test ModuleName
            If p.MainModule.ModuleName.ToLower.Contains("explorer.exe") Then
                p.Kill()
            End If
        Next
        ' ---------------------------------------------------- Not necessary because explorer is restarted automatically 
        ' ---------------------------------------------------- With this command Explorer opens an unwanted window
        'Process.Start("explorer.exe")
    End Sub

    ' =====================================================================================================
    '  TurnON scroll-inactive-windows 
    ' =====================================================================================================
    Friend Sub ScrollInactiveWindows()

        ' ---------------------------------------------------------  (not working)

        ' SCROLL INACTIVE WINDOWS
        ' https://www.tenforums.com/tutorials/12816-turn-off-scroll-inactive-windows-windows-10-a.html

        '  TURN ON
        '[HKEY_CURRENT_USER\Control Panel\Desktop]
        '"MouseWheelRouting"=dword:00000002

        '  TURN OFF
        '[HKEY_CURRENT_USER\Control Panel\Desktop]
        '"MouseWheelRouting"=dword:00000000

        'Dim key As String = "HKEY_CURRENT_USER\Control Panel\Desktop"
        'Dim name As String = "MouseWheelRouting"
        'My.Computer.Registry.SetValue(key, name, 0, Microsoft.Win32.RegistryValueKind.DWord)
        'ShowMessage(" Operation completed succesfully." + vbCrLf + _
        '            " The mouse wheel will scroll inactive windows")
        'RestartExplorerQuestion()


        ' ---------------------------------------------------------  (not working)
        'SystemParametersInfo(SPI_SETMOUSEWHEELROUTING, _
        '                    0, _
        '                    2, _
        '                    SPIF_UPDATEINIFILE Or SPIF_SENDCHANGE)

    End Sub


    ' =====================================================================================================
    '  DISABLE LOCK SCREEN --- (NOT WORKING)
    ' =====================================================================================================
    Friend Sub DisableLockScreen()
        RestartArguments = "DisableLockScreen"
        If Not EnsureAdminPrivileges() Then Return
        '
        Dim key As String
        Dim name As String
        ' ------------------------------------------- old version
        key = "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\Microsoft\Windows\Personalization"
        name = "NoLockScreen"
        My.Computer.Registry.SetValue(key, name, 1, Microsoft.Win32.RegistryValueKind.DWord)
        ' ------------------------------------------- new version
        key = "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\System"
        name = "EnableFirstLogonAnimation"
        My.Computer.Registry.SetValue(key, name, 0, Microsoft.Win32.RegistryValueKind.DWord)
        '
        ShowMessage(" Operation completed succesfully." + vbCrLf + vbCrLf + _
                    " Now the lock screen at logon is disabled.")
    End Sub


    ' =====================================================================================================
    '  RESET SERIAL PORT NUMBERS
    ' =====================================================================================================
    Friend Sub ResetSerialPortNumbers()
        '
        Dim fname As String = Application.StartupPath + "\ComNameArbiterTool\"
        If IsWin64() Then
            fname += "ComNameArbiterTool_64.exe"
        Else
            fname += "ComNameArbiterTool_32.exe"
        End If
        If Not IO.File.Exists(fname) Then
            ShowMessage(" Can not find the application ComNameArbiterTool." + vbCrLf + vbCrLf + _
                        " Verify that the 32 and 64 bit applications are" + vbCrLf + _
                        " in the ""ComNameArbiterTool"" folder.")
            Return
        End If
        '
        RestartArguments = "ResetSerialPortNumbers"
        If Not EnsureAdminPrivileges() Then Return
        If IO.File.Exists(fname) Then Process.Start(fname)
    End Sub

    Friend Function IsWin64() As Boolean
        If IO.Directory.Exists("c:\windows\syswow64") And IO.Directory.Exists("c:\program files (x86)") Then
            Return True
        Else
            Return False
        End If
    End Function

    ' =====================================================================================================
    '  DELETE DESKTOP ICONS FILES
    ' =====================================================================================================
    Friend Sub DeleteDesktoptIconFiles()
        ' --------------------------------------------------------------- Manual test
        ' Command Prompt - Run as administrator.
        ' cd %homepath%\AppData\Local\Microsoft\Windows\Explorer
        ' dir iconcache*
        ' taskkill /f /im explorer.exe
        ' del iconcache*
        ' dir iconcache*
        ' explorer.exe
        ' ---------------------------------------------------------------
        '
        RestartArguments = "DeleteDesktoptIconFiles"
        If Not EnsureAdminPrivileges() Then Return
        '
        Dim processStartInfo As New ProcessStartInfo()
        processStartInfo.FileName = "cmd.exe"
        processStartInfo.Arguments = "/c taskkill /f /im explorer.exe "
        processStartInfo.Arguments += " && cd %homepath%\AppData\Local\Microsoft\Windows\Explorer"
        processStartInfo.Arguments += " && del iconcache*"
        processStartInfo.Arguments += " && explorer.exe"
        processStartInfo.UseShellExecute = False
        processStartInfo.RedirectStandardOutput = True
        Dim proc As New Process()
        proc.StartInfo = processStartInfo
        proc.Start()
        '
        ShowMessage(" Operation completed succesfully." + vbCrLf + vbCrLf + _
                    " The icon files has been deleted.")
    End Sub

End Module
